# pylint: disable-msg=W0622
"""cubicweb-card application packaging information"""

modname = "cubicweb_card"
distname = "cubicweb-card"

numversion = (2, 0, 0)
version = ".".join(str(num) for num in numversion)

license = "LGPL"
author = "LOGILAB S.A. (Paris, FRANCE)"
author_email = "contact@logilab.fr"
web = f"https://forge.extranet.logilab.fr/cubicweb/cubes/{distname}"
description = "card/wiki component for the CubicWeb framework"
classifiers = [
    "Environment :: Web Environment",
    "Framework :: CubicWeb",
    "Programming Language :: Python",
    "Programming Language :: JavaScript",
]

__depends__ = {
    "cubicweb": ">= 4.0.0, < 5.0.0",
    "cubicweb-web": ">= 1.0.0, < 2.0.0",
    "docutils": None,
}
__recommends__ = {
    "cubicweb-preview": None,
    "cubicweb-seo": None,
}



"Card unit test"
import re
import unittest

from cubicweb.devtools import BASE_URL
from cubicweb.devtools.testlib import MAILBOX
from cubicweb_web.devtools.testlib import WebCWTC
from cubicweb_web.ext.rest import rest_publish


class CardTests(WebCWTC):
    def test_notifications(self):
        with self.admin_access.client_cnx() as cnx:
            cnx.create_entity(
                "Card", title="sample card", synopsis="this is a sample card"
            )
            self.assertEqual(len(MAILBOX), 0)
            cnx.commit()
        self.assertEqual(len(MAILBOX), 1)
        self.assertEqual(
            re.sub(r"#\d+", "#EID", MAILBOX[0].subject), "New Card #EID (admin)"
        )


class RestTC(WebCWTC):
    def context(self, cnx):
        return cnx.execute('CWUser X WHERE X login "admin"').get_entity(0, 0)

    def test_card_role_create(self):
        with self.admin_access.client_cnx() as cnx:
            self.assertEqual(
                rest_publish(self.context(cnx), ":card:`index`"),
                f'<p><a class="doesnotexist reference" '
                f'href="{BASE_URL}card/index">index</a></p>\n',
            )

    def test_card_role_create_subpage(self):
        with self.admin_access.client_cnx() as cnx:
            self.assertEqual(
                rest_publish(self.context(cnx), ":card:`foo/bar`"),
                f'<p><a class="doesnotexist reference" '
                f'href="{BASE_URL}card/foo/bar">foo/bar</a></p>\n',
            )  # noqa

    def test_card_role_link(self):
        with self.admin_access.client_cnx() as cnx:
            cnx.create_entity(
                "Card", wikiid="index", title="Site index page", synopsis="yo"
            )
            self.assertEqual(
                rest_publish(self.context(cnx), ":card:`index`"),
                f'<p><a class="reference" '
                f'href="{BASE_URL}card/index">index</a></p>\n',
            )

    def test_nocard_create(self):
        with self.admin_access.web_request("card/foobar") as req:
            content = self.app_handle_request(req)
            create_url = req.build_url("add/Card?wikiid=foobar")
            self.assertIn(f' href="{create_url}"'.encode(), content)
            self.assertEqual(req.status_out, 404)


if __name__ == "__main__":
    unittest.main()



# copyright 2013-2023 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

"""test on entities"""

import unittest
from cubicweb_web.devtools.testlib import WebCWTC


class DCTitleTC(WebCWTC):
    def test_nowikiid(self):
        with self.admin_access.client_cnx() as cnx:
            venus = cnx.create_entity("Card", title="Venus", content="a planet")
            self.assertEqual(venus.dc_title(), "Venus")

    def test_wikiid(self):
        with self.admin_access.client_cnx() as cnx:
            venus = cnx.create_entity(
                "Card", title="Venus", content="a planet", wikiid="Planet_Venus"
            )
            self.assertEqual(venus.dc_title(), "Planet_Venus")


class BreadCrumbsTC(WebCWTC):
    def test_onecard_nowikiid(self):
        with self.admin_access.web_request() as req:
            venus = req.create_entity("Card", title="Venus", content="a planet")
            breadcrumbs = venus.cw_adapt_to("IBreadCrumbs").breadcrumbs()
            self.assertListEqual([venus], breadcrumbs)

    def test_onecard(self):
        with self.admin_access.web_request() as req:
            venus = req.create_entity(
                "Card", title="Venus", content="a planet", wikiid="Venus"
            )
            breadcrumbs = venus.cw_adapt_to("IBreadCrumbs").breadcrumbs()
            self.assertListEqual([venus], breadcrumbs)

    def test_twocards(self):
        """Planet and Planet/Venus"""
        with self.admin_access.web_request() as req:
            planet = req.create_entity("Card", title="Planet", wikiid="Planet")
            venus = req.create_entity("Card", title="Venus", wikiid="Planet/Venus")
            breadcrumbs = venus.cw_adapt_to("IBreadCrumbs").breadcrumbs()
            self.assertListEqual([planet, venus], breadcrumbs)

    def test_onecardnested(self):
        """Planet/Venus without Planet"""
        with self.admin_access.web_request() as req:
            venus = req.create_entity("Card", title="Venus", wikiid="Planet/Venus")
            breadcrumbs = venus.cw_adapt_to("IBreadCrumbs").breadcrumbs()
            self.assertListEqual(["Planet", venus], breadcrumbs[1:])

    def test_treecards_oneinexistent(self):
        """Astronomy and Astronomy/Planet/Venus (no Astronomy/Planet)"""
        with self.admin_access.web_request() as req:
            astro = req.create_entity("Card", title="Astronomy", wikiid="Astronomy")
            venus = req.create_entity(
                "Card", title="Venus", wikiid="Astronomy/Planet/Venus"
            )
            breadcrumbs = venus.cw_adapt_to("IBreadCrumbs").breadcrumbs()
            self.assertListEqual([astro, "Planet", venus], breadcrumbs)

    def test_treecards_oneinexistent2(self):
        """Astronomy/Planet and Astronomy/Planet/Venus (no Astronomy)"""
        with self.admin_access.web_request() as req:
            planet = req.create_entity(
                "Card", title="Planets", wikiid="Astronomy/Planet"
            )
            venus = req.create_entity(
                "Card", title="Venus", wikiid="Astronomy/Planet/Venus"
            )
            breadcrumbs = venus.cw_adapt_to("IBreadCrumbs").breadcrumbs()
            self.assertListEqual(["Astronomy", planet, venus], breadcrumbs[1:])


if __name__ == "__main__":
    unittest.main()



"""template automatic tests"""

import unittest

from cubicweb_web.devtools.testlib import WebCWTC


class AutomaticWebTest(WebCWTC):
    def to_test_etypes(self):
        return {"Card"}


class CardTC(WebCWTC):
    def setup_database(self):
        with self.admin_access.repo_cnx() as cnx:
            self.card = cnx.create_entity(
                "Card", title="sample card", synopsis="this is a sample card"
            ).eid
            cnx.commit()

    def test_views(self):
        with self.admin_access.web_request() as req:
            fobj = req.entity_from_eid(self.card)
            self.vreg["views"].select_or_none("inlined", fobj._cw, rset=fobj.cw_rset)


if __name__ == "__main__":
    unittest.main()



from docutils import nodes, utils
from docutils.parsers.rst.roles import register_canonical_role, set_classes


def card_reference_role(role, rawtext, text, lineno, inliner, options={}, content=[]):
    text = text.strip()
    try:
        wikiid, rest = text.split(":", 1)
    except Exception:
        wikiid, rest = text, text
    context = inliner.document.settings.context
    ref = context._cw.build_url("card/" + wikiid)
    rset = context._cw.execute("Card C WHERE C wikiid %(w)s", {"w": wikiid})
    set_classes(options)
    if not rset:
        options["classes"] = ["doesnotexist"]
    else:
        options.pop("classes", None)
    return [nodes.reference(rawtext, utils.unescape(rest), refuri=ref, **options)], []


register_canonical_role("card", card_reference_role)



"""cube-specific forms/views/actions/components

Specific views for cards

:organization: Logilab
:copyright: 2001-2014 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
:contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
"""
__docformat__ = "restructuredtext en"

from cubicweb import _


from logilab.mtconverter import xml_escape
from logilab.common.decorators import cachedproperty

from cubicweb.predicates import is_instance, match_form_params
from cubicweb_web.views import uicfg, primary, baseviews
from cubicweb_web.views.autoform import AutomaticEntityForm
from cubicweb_web.views.ibreadcrumbs import IBreadCrumbsAdapter

uicfg.primaryview_section.tag_attribute(("Card", "title"), "hidden")
uicfg.primaryview_section.tag_attribute(("Card", "synopsis"), "hidden")
uicfg.primaryview_section.tag_attribute(("Card", "wikiid"), "hidden")


class CardPrimaryView(primary.PrimaryView):
    __select__ = is_instance("Card")
    show_attr_label = False

    def render_entity_title(self, entity):
        self.w(f"<h1>{xml_escape(entity.title)}</h1>")
        if entity.synopsis:
            self.w(f"<div class=\"summary\">{entity.printable_value('synopsis')}</div>")


class CardInlinedView(CardPrimaryView):
    """hide card title and summary"""

    __regid__ = "inlined"
    title = _("inlined view")
    main_related_section = False

    def render_entity_title(self, entity):
        self.w(f"<div class=\"summary\">{entity.printable_value('synopsis')}</div>")

    def content_navigation_components(self, context):
        pass


class CardDoesNotExistView(baseviews.NoResultView):
    __select__ = baseviews.NoResultView.__select__ & match_form_params("wikiid")

    def call(self, **kwargs):
        super().call(**kwargs)
        etype = self._cw.vreg["etypes"].etype_class("Card")
        ctx = {
            "url": etype.cw_create_url(self._cw, wikiid=self._cw.form["wikiid"]),
            "message": xml_escape(self._cw._("This card does not exist yet.")),
            "invite": xml_escape(self._cw._("Create it?")),
            "notice": "",
        }
        if not self._cw.vreg.schema["Card"].has_perm(self._cw, "add"):
            ctx["notice"] = xml_escape(self._cw._(" (You may need to log in first.)"))
        self.w(
            '<div class="section">%(message)s '
            '<a href="%(url)s">%(invite)s</a>%(notice)s</div>' % ctx
        )
        self._cw.status_out = 404


class CardBreadCrumbsAdapter(IBreadCrumbsAdapter):
    __select__ = IBreadCrumbsAdapter.__select__ & is_instance("Card")

    @cachedproperty
    def dirname(self):
        if self.entity.wikiid:
            return "/".join(self.entity.wikiid.split("/")[:-1])

    def card_from_wikiid(self, path):
        """Return the Card given its ``wikiid`` path or None"""
        rset = self._cw.execute("Card C WHERE C wikiid %(id)s", {"id": path})
        if rset:
            return rset.get_entity(0, 0)

    def parent_entity(self):
        if not self.dirname:
            # card is at root: return whatever super returns
            parent = super().parent_entity()
        else:
            parent = self.card_from_wikiid(self.dirname)
        return parent

    def breadcrumbs(self, view=None, recurs=None):
        """Virtual hierarchy of wiki pages following a directory-like
        structure.
        """
        if self.parent_entity() or not self.dirname:
            # parent Card exists or the current Card is at the root (i.e. its
            # wikiid has no /)
            return super().breadcrumbs(view, recurs)
        else:
            # parent card does not exist: build the (reversed) path by
            # iterating on the directory structure
            path = [self.entity]
            dirs = self.dirname.split("/")
            while dirs:
                card = self.card_from_wikiid("/".join(dirs))
                p = [dirs.pop()]
                if card:
                    p = card.cw_adapt_to("IBreadCrumbs").breadcrumbs(view, recurs)
                path.extend(p)
            if not card:
                # last path entry is not a card
                path.append(
                    (
                        self._cw.build_url(str(self.entity.e_schema)),
                        self._cw._("Card_plural"),
                    )
                )
            path.reverse()
            return path


try:
    from cubicweb_seo.views import SitemapRule

    class CardSitemapRule(SitemapRule):
        __regid__ = "card"
        query = "Any X WHERE X is Card"
        priority = 1.0

except ImportError:
    pass


def registration_callback(vreg):
    vreg.register(CardPrimaryView)
    vreg.register(CardInlinedView)
    vreg.register(CardDoesNotExistView)
    vreg.register(CardBreadCrumbsAdapter)

    loaded_cubes = vreg.config.cubes()

    if "seo" in loaded_cubes:
        vreg.register(CardSitemapRule)

    if "preview" in loaded_cubes:
        from cubicweb_preview.views.forms import PreviewFormMixin

        class PreviewAutomaticEntityForm(PreviewFormMixin, AutomaticEntityForm):
            preview_mode = "inline"
            __select__ = AutomaticEntityForm.__select__ & is_instance("Card")

        vreg.register(PreviewAutomaticEntityForm)



from cubicweb.predicates import match_form_params
from cubicweb_web.views import basecontrollers


class CardViewController(basecontrollers.ViewController):
    __regid__ = "card"
    __select__ = basecontrollers.ViewController.__select__ & match_form_params("wikiid")

    def publish(self, rset=None):
        rset = self._cw.execute("Card C WHERE C wikiid %(wikiid)s", self._cw.form)
        if rset:
            # Drop wikiid from form to get the standard behaviour.
            # Otherwise, the presence of wikiid in form ensures that
            # CardDoesNotExistView is selected.
            self._cw.form.pop("wikiid")
        return super().publish(rset)



"""Card notification hooks

:organization: Logilab
:copyright: 2003-2017 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
:contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

from cubicweb.predicates import is_instance
from cubicweb.sobjects.notification import ContentAddedView


class CardAddedView(ContentAddedView):
    """get notified from new cards"""

    __select__ = is_instance("Card")
    content_attr = "synopsis"



"""this contains the cube-specific entities' classes"""

from cubicweb.entities import AnyEntity, fetch_config


class Card(AnyEntity):
    """customized class for Card entities"""

    __regid__ = "Card"
    rest_attr = "wikiid"

    fetch_attrs, cw_fetch_order = fetch_config(["title"])

    def dc_title(self):
        if self.wikiid:
            return self.wikiid.split("/")[-1]
        else:
            return self.title

    def dc_description(self, format="text/plain"):
        return self.printable_value("synopsis", format=format)

    def rest_path(self):
        if self.wikiid:
            return "{}/{}".format(
                str(self.e_schema).lower(),
                self._cw.url_quote(self.wikiid, safe="/"),
            )
        else:
            return super().rest_path()



#!/usr/bin/env python
# pylint: disable=W0142,W0403,W0404,W0613,W0622,W0622,W0704,R0904,C0103,E0611
#
# CubicWeb is free software: you can redistribute it and/or modify it under the
# terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# CubicWeb is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with CubicWeb.  If not, see <http://www.gnu.org/licenses/>.
"""cubicweb_francearchives setup module using data from
cubicweb_card/__pkginfo__.py file
"""

from os.path import join, dirname
from setuptools import find_packages, setup


here = dirname(__file__)

# load metadata from the __pkginfo__.py file so there is no risk of conflict
# see https://packaging.python.org/en/latest/single_source_version.html
pkginfo = join(here, "cubicweb_card", "__pkginfo__.py")
__pkginfo__ = {}
with open(pkginfo) as f:
    exec(f.read(), __pkginfo__)

# get required metadatas
distname = __pkginfo__["distname"]
version = __pkginfo__["version"]
license = __pkginfo__["license"]
description = __pkginfo__["description"]
web = __pkginfo__["web"]
author = __pkginfo__["author"]
author_email = __pkginfo__["author_email"]
classifiers = __pkginfo__["classifiers"]

with open(join(here, "README.rst")) as f:
    long_description = f.read()

# get optional metadatas
dependency_links = __pkginfo__.get("dependency_links", ())

requires = {}
for entry in ("__depends__",):  # "__recommends__"):
    requires.update(__pkginfo__.get(entry, {}))
install_requires = [f"{d} {v and v or ''}".strip() for d, v in requires.items()]


setup(
    name=distname,
    version=version,
    license=license,
    description=description,
    long_description=long_description,
    author=author,
    author_email=author_email,
    url=web,
    classifiers=classifiers,
    packages=find_packages(exclude=["test"]),
    install_requires=install_requires,
    include_package_data=True,
    entry_points={
        "cubicweb.cubes": [
            "card=cubicweb_card",
        ],
        "cubicweb.i18ncube": [
            "card=cubicweb_card.i18n",
        ],
    },
    zip_safe=False,
)
